@@ -95,7 +95,7 @@ public:
         return std::shared_ptr<BinaryToReal>(new BinaryToReal(create));
     }
 
-    static std::shared_ptr<BinaryToReal> Create(index_t frame_integration_size=1, indices_t output_shape = indices_t())
+    static std::shared_ptr<BinaryToReal> Create(index_t frame_integration_size, indices_t output_shape = indices_t())
     {
         create_t create;
         create.output_shape    = output_shape;
@@ -103,12 +103,22 @@ public:
         return Create(create);
     }
 
+    static std::shared_ptr<BinaryToReal> Create(index_t frame_integration_size=1, index_t depth_integration_size=0, indices_t output_shape = indices_t())
+    {
+        create_t create;
+        create.output_shape    = output_shape;
+        create.frame_integration_size = frame_integration_size;
+        create.depth_integration_size = depth_integration_size;
+        return Create(create);
+    }
+
 #ifdef BB_PYBIND11
     static std::shared_ptr<BinaryToReal> CreatePy(index_t frame_integration_size=1, index_t depth_integration_size=0, indices_t output_shape = indices_t())
     {
         create_t create;
         create.output_shape    = output_shape;
         create.frame_integration_size = frame_integration_size;
+        create.depth_integration_size = depth_integration_size;
         return Create(create);
     }
 #endif
@@ -269,6 +279,10 @@ public:
 
     FrameBuffer Backward(FrameBuffer dy_buf) override
     {
+        if (dy_buf.Empty()) {
+            return dy_buf;
+        }
+
         if ( !m_binary_mode || (m_frame_integration_size == 1 && m_input_shape == m_output_shape) ) {
             return dy_buf;
         }
