@@ -258,6 +258,10 @@ public:
      */
     inline FrameBuffer Backward(FrameBuffer dy_buf)
     {
+        if (dy_buf.Empty()) {
+            return dy_buf;
+        }
+
         BB_ASSERT(dy_buf.GetType() == DataType<BT>::type);
 
         // 戻り値のサイズ設定
