@@ -14,7 +14,9 @@ import binarybrain.core as core
 #core_model_list = bb.get_core_subclass_dict(core.Model)
 #core_model_dict = bb.get_core_subclass_dict(core.Model)
 
-def search_core_model(model_name, dtypes=[]):
+def search_core_model(model_name, dtypes=None):
+    if dtypes is None:
+        dtypes = []
     return bb.search_core_object(model_name, dtypes)
 
 _model_creator_list = {}
@@ -308,61 +310,6 @@ class Model(bb.Object):
         if core_model is not None:
             return bb.FrameBuffer.from_core(core_model.backward(dy_buf.get_core()))
         return dy_buf
-    
-    ''' objectクラスに移管
-    def dumps(self) -> bytes:
-        """バイトデータにシリアライズ
-
-           モデルのデータをシリアライズして保存するためのバイト配列を生成
-        
-        Returns:
-            data (bytes): Serialize data
-        """
-        core_model = self.get_core()
-        if core_model is not None:
-            return core_model.dump_object()
-        return b''
-        
-    def loads(self, data: bytes) -> bytes:
-        """バイトデータをロード
-
-           モデルのデータをシリアライズして復帰のバイト配列ロード
-        
-        Args:
-            data (bytes): Serialize data
-        """
-        core_model = self.get_core()
-        if core_model is not None:
-            size = core_model.load_object(data)
-            return data[size:]
-        return data
-
-    def dump(self, filename: str):
-        """ファイルに保存
-
-           モデルのデータをシリアライズしてファイルに保存
-        
-        Args:
-            filename (str): ファイル名
-        """
-        with open(filename, 'wb') as f:
-            f.write(self.dumps())
-
-
-    def load(self, filename: str):
-        """ファイルからロード
-
-           ファイルからロード
-        
-        Args:
-            filename (str): ファイル名
-        """
-        with open(filename, 'rb') as f:
-            data, obj = self.loads(f.read())
-            if data != b'':
-                print('[Model.loads] warrning: data is too long')
-            return obj
-    '''
 
     def dump_bytes(self):
         # バイトデータにシリアライズ(old format)
@@ -378,6 +325,9 @@ class Model(bb.Object):
             size = core_model.load(data)
             return data[size:]
         return data
+    
+    def __str__(self):
+        return self.get_info(1)
 
 
     
@@ -397,7 +347,10 @@ class Sequential(Model):
         model_list (List[Model]): モデルのリスト
     """
     
-    def __init__(self, model_list=[], *, input_shape=None, name=None):
+    def __init__(self, model_list=None, *, input_shape=None, name=None):
+        if model_list is None:
+            model_list = []
+        self.name = ''
         self.model_list = model_list
         super(Sequential, self).__init__(input_shape=input_shape, name=name)
 
@@ -410,6 +363,21 @@ class Sequential(Model):
             core_model.set_name(self.name)            
         return core_model
     
+    def set_name(self, name):
+        self.name = name
+
+    def get_name(self):
+        if self.name is None or len(self.name) == 0:
+            return self.get_model_name()
+        return self.name
+
+    def is_named(self):
+        if self.name is None:
+            return False
+        if len(self.name) == 0: 
+            return False
+        return True
+
     def set_model_list(self, model_list):
         """モデルリストの設定
        
@@ -439,24 +407,52 @@ class Sequential(Model):
         self.model_list[item] = model
     
     def append(self, model):
-        """モデルリストの追加
+        """リストへのモデル追加
        
-        Returns:
+        Args:
             model (Model): リストに追加するモデル
         """
         self.model_list.append(model)
     
+    def remove(self, model):
+        """リストへからモデル削除
+       
+        Args:
+            model (Model): リストから削除するモデル
+        """
+        self.model_list.remove(model)
+
+
     def get_info(self, depth=0, *, columns=70, nest=0):
         # これ以上ネストしないなら自クラス概要
         if depth > 0 and (nest+1) >= depth:
             return super(Sequential, self).get_info(depth=depth, columns=columns, nest=nest)
         else:
-            # 子レイヤー
-            info = ''
-            for model in self.model_list:
-                info += model.get_info(depth=depth, columns=columns, nest=nest+1)
-            return info
+        # セパレータとインデント文字列生成
+            indent    = ' ' * (nest*2)
+            separetor = '-' * (columns - len(indent))
+            name      = self.name
+            if name is None:
+                name = ''
 
+            # モデルタイトル
+            text  = indent + separetor + '\n'
+            text += indent + '[' + self.get_model_name() + '] ' + name + '\n'
+
+            # 内容
+            text += indent + ' input  shape : ' + str(self.get_input_shape())
+            text += indent + ' output shape : ' + str(self.get_output_shape()) + '\n'
+
+            # 子レイヤー
+            for model in self.model_list:
+                text += model.get_info(depth=depth, columns=columns, nest=nest+1)
+
+            # 最上段なら末尾セパレータ追加
+            if nest == 0:
+                text +=  indent + separetor + '\n'
+            return text
+
+    
     def send_command(self, command, send_to="all"):
         for model in self.model_list:
             model.send_command(command=command, send_to=send_to)
@@ -496,9 +492,15 @@ class Sequential(Model):
         data = bb.dump_object_header('Sequential')
         
         # バージョン
-        ver = 1
+        ver = 2
         data += bb.int_to_bytes(ver)
         
+        # 名前
+        name = self.name
+        if name is None:
+            name = ''
+        data += bb.string_to_bytes(name)
+
         # レイヤ数
         layer_size = len(self.model_list)
         data += bb.int_to_bytes(layer_size)
@@ -516,8 +518,13 @@ class Sequential(Model):
         
         # バージョン
         data, ver = bb.int_from_bytes(data)
-        assert(ver == 1)
+        assert(ver == 1 or ver == 2)
         
+        if ver == 2:
+            data, name = bb.string_from_bytes(data)
+            if not self.is_named():
+                self.name = name
+
         # レイヤ数
         data, layer_size = bb.int_from_bytes(data)
 
@@ -536,7 +543,8 @@ class Sequential(Model):
 
     @classmethod
     def from_bytes(cls, data):
-        new_model = cls(model_list=None)
+        new_model = cls()
+        new_model.model_list=None
         data = new_model.loads(data)
         return data, new_model
 
@@ -555,7 +563,9 @@ class Switcher(Model):
         init_model_name (str): 初期選択するモデルの名前
     """
     
-    def __init__(self, model_dict={}, init_model_name=None, *, input_shape=None, name=None):
+    def __init__(self, model_dict=None, init_model_name=None, *, input_shape=None, name=None):
+        if model_dict is None:
+            model_dict = {}
         self.model_dict = model_dict
         self.current_model_name = None
         self.current_model = None
@@ -573,6 +583,12 @@ class Switcher(Model):
                 self.current_model_name = model_name
                 self.current_model      = self.model_dict[model_name]
     
+    def __getitem__(self, key):
+        return self.model_dict[key]
+
+    def __setitem__(self, key, value):
+        self.model_dict[key] = value
+
     def get_model_list(self):
         return self.model_dict.values()
     
@@ -714,7 +730,8 @@ class Switcher(Model):
 
     @classmethod
     def from_bytes(cls, data):
-        new_model = cls(model_dict=None)
+        new_model = cls()
+        new_model.model_dict=None
         data = new_model.loads(data)
         return data, new_model
 
@@ -764,8 +781,10 @@ class BinaryToReal(Model):
         bin_dtype (DType): 入力の型を bb.DType.FP32 もしくは bb.DType.BIT で指定可能
     """
     
-    def __init__(self, *, frame_integration_size=1, depth_integration_size=1, output_shape=[], input_shape=None, name=None,
+    def __init__(self, *, frame_integration_size=1, depth_integration_size=1, output_shape=None, input_shape=None, name=None,
                                                     bin_dtype=bb.DType.FP32, real_type=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('BinaryToReal', [bin_dtype, real_type]).create
             core_model = core_creator(frame_integration_size=frame_integration_size, depth_integration_size=depth_integration_size, output_shape=output_shape)
@@ -784,8 +803,10 @@ class BitEncode(Model):
         bit_size (int): エンコードするbit数
     """
 
-    def __init__(self, bit_size=1, *, output_shape=[], input_shape=None, name=None,
+    def __init__(self, bit_size=1, *, output_shape=None, input_shape=None, name=None,
                         bin_dtype=bb.DType.FP32, real_dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('BitEncode', [bin_dtype, real_dtype]).create
             core_model = core_creator(bit_size, output_shape)
@@ -802,13 +823,16 @@ class Reduce(Model):
 
     Args:
         output_shape ([int]): 出力のシェイプ
+        integration_size (int): 積算するサイズ
     """
 
-    def __init__(self, output_shape=[], integrate_size=0, *, input_shape=None, name=None,
-                        fw_dtype=bb.DType.FP32, bw_dtype=bb.DType.FP32, core_model=None):
+    def __init__(self, output_shape=None, integration_size=0, *, input_shape=None, name=None,
+                        bin_dtype=bb.DType.FP32, real_dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
-            core_creator = search_core_model('Reduce', [fw_dtype, bw_dtype]).create
-            core_model = core_creator(output_shape, integrate_size)
+            core_creator = search_core_model('Reduce', [bin_dtype, real_dtype]).create
+            core_model = core_creator(output_shape, integration_size)
         
         super(Reduce, self).__init__(core_model=core_model, input_shape=input_shape, name=name)
 
@@ -829,8 +853,10 @@ class DenseAffine(Model):
         seed (int): 変数初期値などの乱数シード
     """
     
-    def __init__(self, output_shape=[], *, input_shape=None, initialize_std=0.01, initializer='he', seed=1, name=None,
+    def __init__(self, output_shape=None, *, input_shape=None, initialize_std=0.01, initializer='he', seed=1, name=None,
                         dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('DenseAffine', [dtype]).create
             core_model = core_creator(output_shape=output_shape, initialize_std=initialize_std, initializer=initializer, seed=seed)
@@ -854,8 +880,10 @@ class DepthwiseDenseAffine(Model):
         seed (int): 変数初期値などの乱数シード
     """
     
-    def __init__(self, output_shape=[], *, input_shape=None, input_point_size=0, depth_size=0,
+    def __init__(self, output_shape=None, *, input_shape=None, input_point_size=0, depth_size=0,
                             initialize_std=0.01, initializer='he', seed=1, name=None, dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('DepthwiseDenseAffine', [dtype]).create
             core_model = core_creator(output_shape=output_shape, initialize_std=initialize_std, initializer=initializer, seed=seed)
@@ -965,9 +993,11 @@ class DifferentiableLut(SparseModel):
         bin_dtype (DType)): バイナリ出力の型を bb.DType.FP32 と bb.DType.BIT から指定(bb.DType.BIT は binarize=True 時のみ)
     """
     
-    def __init__(self, output_shape=[], *, input_shape=None,
+    def __init__(self, output_shape=None, *, input_shape=None,
                     connection='random', binarize=True, batch_norm=True, momentum=0.0, gamma= 0.3, beta=0.5, seed=1,
                     name=None, N=6, bin_dtype=bb.DType.FP32, real_dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('DifferentiableLut' + str(N), [bin_dtype, real_dtype]).create
             core_model = core_creator(output_shape, batch_norm, binarize, connection, momentum, gamma, beta, seed)
@@ -1028,8 +1058,10 @@ class BinaryLut(SparseModel):
         fw_dtype (DType)): forwardの型を bb.DType.FP32 と bb.DType.BIT から指定
     """
     
-    def __init__(self, output_shape=[], *, input_shape=None,
+    def __init__(self, output_shape=None, *, input_shape=None,
                     connection='random', seed=1, name=None, N=6, fw_dtype=bb.DType.FP32, bw_dtype=bb.DType.FP32, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('BinaryLut' + str(N), [fw_dtype, bw_dtype]).create
             core_model  = core_creator(output_shape, connection, seed)
@@ -1428,6 +1460,22 @@ class HardTanh(Model):
 model_creator_regist('HardTanh', HardTanh.from_bytes)
 
 
+
+class Softmax(Model):
+    """Softmax class
+
+       Softmax 活性化層
+    """
+    def __init__(self, *, input_shape=None, name=None, dtype=bb.DType.FP32, core_model=None):
+        if core_model is None:
+            core_creator = search_core_model('Softmax', [dtype]).create
+            core_model = core_creator()
+
+        super(Softmax, self).__init__(core_model=core_model, input_shape=input_shape, name=name)
+
+model_creator_regist('Softmax', Softmax.from_bytes)
+
+
 # ------- 補助モデル --------
 
 class BatchNormalization(Model):
@@ -1484,7 +1532,9 @@ class Shuffle(Model):
         shuffle_unit (int): シャッフルする単位
     """
 
-    def __init__(self, shuffle_unit, *, output_shape=[], input_shape=None, name=None, core_model=None):
+    def __init__(self, shuffle_unit, *, output_shape=None, input_shape=None, name=None, core_model=None):
+        if output_shape is None:
+            output_shape = []
         if core_model is None:
             core_creator = search_core_model('Shuffle', []).create
             core_model = core_creator(shuffle_unit, output_shape)
@@ -1526,3 +1576,24 @@ def get_model_list(net, flatten:bool =False):
     flatten_list(net, out_list)
     
     return out_list
+
+
+def get_model_list_for_rtl(net):
+    # RTL化用リストを得る
+    
+    if  type(net) is not list:
+        net = [net]
+    
+    def flatten_list(in_list, out_list):
+        for model in in_list:
+            if issubclass(type(model), Switcher):
+                model = model.get_current_model()
+            if type(model) != Convolution2d and hasattr(model, 'get_model_list'):
+                flatten_list(model.get_model_list(), out_list)
+            else:
+                out_list.append(model)
+    
+    out_list = []
+    flatten_list(net, out_list)
+    
+    return out_list
